# SPDX-FileCopyrightText: 2024-present Sebastian Tobie <sebastian@sebastian-tobie.de>
#
# SPDX-License-Identifier: MIT
from gunicorn.glogging import Logger as gLogger
from logging import Formatter
from logging.config import dictConfig
from logging.config import fileConfig
import os.path

__version__ = "0.0.1"


class Logger(gLogger):
    def __init__(self, cfg):
        self.error_log = logging.getLogger("gunicorn.error")
        self.access_log = logging.getLogger("gunicorn.access")
        self.logfile = None
        self.lock = threading.Lock()
        self.cfg = cfg
        self.setup(cfg)

    def setup(self, cfg):
        if cfg.logconfig_dict:
            config = CONFIG_DEFAULTS.copy()
            config.update(cfg.logconfig_dict)
            try:
                dictConfig(config)
            except (AttributeError, ImportError, ValueError, TypeError) as exc:
                raise RuntimeError(str(exc))
        elif cfg.logconfig_json:
            config = CONFIG_DEFAULTS.copy()
            if os.path.exists(cfg.logconfig_json):
                try:
                    config_json = json.load(open(cfg.logconfig_json))
                    config.update(config_json)
                    dictConfig(config)
                except (json.JSONDecodeError, AttributeError, ImportError, ValueError, TypeError) as exc:
                    raise RuntimeError(str(exc))
        elif cfg.logconfig:
            if os.path.exists(cfg.logconfig):
                defaults = CONFIG_DEFAULTS.copy()
                defaults['__file__'] = cfg.logconfig
                defaults['here'] = os.path.dirname(cfg.logconfig)
                fileConfig(cfg.logconfig, defaults=defaults, disable_existing_loggers=False)
            else:
                msg = "Error: log config '%s' not found"
                raise RuntimeError(msg % cfg.logconfig)

    def atoms(self, resp, req, environ, request_time):
        atoms = super().atoms(resp, req, environ, request_time)
        atoms["extra"] = dict(
            method=environ.get('REQUEST_METHOD'),
            path=environ.get('PATH_INFO'),
            query=environ.get('QUERY_STRING'),
            proto=environ.get('SERVER_PROTOCOL'),
            time="%d.%06d" % (request_time.seconds, request_time.microseconds),
        )
        if environ.get('REMOTE_ADDR', False):
            atoms["extra"]["remote"] = environ.get('REMOTE_ADDR')
        if getattr(resp, 'sent', False):
            atoms["extra"]["length"] = getattr(resp, 'sent')
        if environ.get('HTTP_REFERER', False):
            atoms["extra"]["referer"] = environ.get('HTTP_REFERER')
        if environ.get('HTTP_USER_AGENT', False):
            atoms["extra"]["user_agent"] = environ.get('HTTP_USER_AGENT')
        return atoms
